# WARNING: Please don't edit this file. It was generated by Python/WinRT v3.0.0

import enum
import typing
import uuid as _uuid

import winrt.system
from winrt._winrt_windows_management_setup import (
    AgentProvisioningProgressReport,
    DeploymentSessionConnectionChangedEventArgs,
    DeploymentSessionHeartbeatRequestedEventArgs,
    DeploymentSessionStateChangedEventArgs,
    DeploymentWorkload,
    DeploymentWorkloadBatch,
    DevicePreparationExecutionContext,
    MachineProvisioningProgressReporter,
)

__all__ = [
    "DeploymentAgentProgressState",
    "DeploymentSessionConnectionChange",
    "DeploymentSessionStateChange",
    "DeploymentWorkloadState",
    "AgentProvisioningProgressReport",
    "DeploymentSessionConnectionChangedEventArgs",
    "DeploymentSessionHeartbeatRequestedEventArgs",
    "DeploymentSessionStateChangedEventArgs",
    "DeploymentWorkload",
    "DeploymentWorkloadBatch",
    "DevicePreparationExecutionContext",
    "MachineProvisioningProgressReporter",
    "DeploymentSessionHeartbeatRequested",
]

class DeploymentAgentProgressState(enum.IntEnum):
    NOT_STARTED = 0
    INITIALIZING = 1
    IN_PROGRESS = 2
    COMPLETED = 3
    ERROR_OCCURRED = 4
    REBOOT_REQUIRED = 5
    CANCELED = 6

class DeploymentSessionConnectionChange(enum.IntEnum):
    NO_CHANGE = 0
    HOST_CONNECTION_LOST = 1
    HOST_CONNECTION_RESTORED = 2
    AGENT_CONNECTION_LOST = 3
    AGENT_CONNECTION_RESTORED = 4
    INTERNET_CONNECTION_LOST = 5
    INTERNET_CONNECTION_RESTORED = 6

class DeploymentSessionStateChange(enum.IntEnum):
    NO_CHANGE = 0
    CANCEL_REQUESTED_BY_USER = 1
    RETRY_REQUESTED_BY_USER = 2

class DeploymentWorkloadState(enum.IntEnum):
    NOT_STARTED = 0
    IN_PROGRESS = 1
    COMPLETED = 2
    FAILED = 3
    CANCELED = 4
    SKIPPED = 5
    UNINSTALLED = 6
    REBOOT_REQUIRED = 7

DeploymentSessionHeartbeatRequested = typing.Callable[[DeploymentSessionHeartbeatRequestedEventArgs], None]
